int DCC_adresse = 3;           // DCC-weichenadresse, ziffer entsprechend der gewuenschten adresse anpassen
int intervall = 100;       // schaltzeit in ms; auch abh. von einstellung zentrale oder steuergeraet
int ausgang1a = 1;   // nano: 11 (z.B.); Attiny85: 0, 1, 3, 4. doppelspule mit darlingtontrieber nicht an pin 0 bei jumper-bootloader. achtung: pin1 eingebaute LED
int ausgang1b = 0;  // nano: 12 (z.B.) 
int weichenlaterne1_adr = 33;  // adresse fuer weichenlaterne oder entkupplersignal
int weichenlaterne1_pin = 4; // led mit vorwid. oder treiber fuer gluehbirnen
int weichenlaterne1_dim = 250; // dimmwert fuer weichenlaterne 0 - 254 (30); PWM-pins nano 3, 5, 6, 9, 10, 11, PWM-pins Attiny85: O, 1, 4

/* DCC-einzelweichedekoder (magnetartikeldecoder) fuer motorische weichenantriebe, z.B. LGB EPL-Antrieb 1201(0) oder boehler (schaltzeit zeile 2 anpassen) mit zuschaltbarer weichenlaterne.
 * alternativ doppelspulenatrieb z.B. LGB 1206 s.u. 
 * DCC-adresse wird in zeile 1 eingestellt. ausgaenge fuer links oder rechtsweiche bzw. je nach verschaltung in zeile 3 & 4 anpassen. 
 * (zwei ausgaenge werden fuer eine weiche benoetigt).
 * automatische zeitgesteuerte rueckfallweiche durch anpassung / ergaenzung der LOW / HIGH - befehle moeglich.
 *  
 * hier vereinfacht fuer einzelweichendekoder mit schaltzeit und fester (in zeile 1 vor hochladen festlegen) weichenadresse.
 * 
 * A.G.-P.
 * 
 * die benoetigte NmraDCC - bibliothek gibt es unter https://downloads.arduino.cc/libraries/github.com/mrrwa/NmraDcc-1.4.2.zip
 * !!! WICHTIG: laeuft nur mit NmraDcc version 1.4.2 Library; ggf. in bibl.-verwaltung entsprechen einstellen!
 * vergleiche: https://github.com/mrrwa/NmraDCC/archive/master.zip
 * siehe auch: https://github.com/mrrwa/NmraDcc
 * bibliothek geschreiben von Alex Shepherd, Wolfgang Kuffer, Geoff Bunza, Martin Pischky, Franz-Peter Mueller, Sven (littleyoda), Hans Tanner
 * infos: https://www.arduino.cc/reference/en/libraries/nmradcc/
 * 
 * hardware:
 * 
 *  - Attiny85 Mini board (Digispark nachbau) mit Digistump/Digispark bootloader 
 *    achtung: je nach bootloader geht das board zunaechst in den programmiermodus (z.B. 5s), wenn es mit spannung versorgt wird.
 *    basis hardware: Karlheinz Marks http://www.robokalle.de/elektronik%20dcc-lichtsignal%20decoder.htm
 *      -- ggf. muss USB-treiber fuer betriebssystem installiert werden https://github.com/digistump/DigistumpArduino/tree/master/tools 
 *         achtung: board hat keinen USB-RS232-wandler, nur virtueller USB, funktioniert nicht mit jedem host! ggf. (alten) USB2-Hub mit 
 *         eigener stromversorgung verwenden https://digistump.com/wiki/digispark/tutorials/connecting
 *         MAC OSX: ~/Library/Arduino15/packages/digistump/hardware/avr/1.6.7/libraries/DigisparkKeyboard/usbconfig.h
 *         ggf. eintrag aendern: #define USB_CFG_VENDOR_ID 0xac, 0x05
 *         vgl.: https://null-byte.wonderhowto.com/how-to/hack-macos-with-digispark-ducky-script-payloads-0198555/
 *      -- Arduino IDE muss fuer Digistump AVR Boards (Digispark 16,5 MHz) ergaenzt werden: 
 *          https://raw.githubusercontent.com/digistump/arduino-boards-index/master/package_digistump_index.json
 *          erlaeuterung: http://blog.wenzlaff.de/?p=7721
 *          hochladen: board "Digispark (Default - 16.5mhz)"; programmer Micronucleus; OSX: keinen port waehlen; OSX, WIN, LINUX: board erst auf anforderung einstecken
 *          falls micronucleus-1.11-entry-jumper-pb0-upgrade.hex installiert: P0 -> GND zum hochladen
 *          https://github.com/overfl0/NocInformatykaBoard/tree/master/upgrade/releases
 *          aktuelle firmware: https://github.com/micronucleus/micronucleus/tree/master/firmware
 *          version mit USB2-kontakten auf platine: auf sicheren kontakt achten
 *      -- P2: DCC command; P3: motorboard IN1; P4 motorboard IN2
 *      -- pin-info: P0: I2C SDA, PWM, pin1: PWM, pin2: I2C SCK, analog in, pin3: analog in, USB+, pin4: PWM, analog in, USB-, pin5: analog in; 
 *         Rev2 / Rev4: eingebaute LED -> pin1. Rev1: eingebaute LED -> pin0 
 *         (this board requires the trace to the LED to be cut (or the LED desoldered) if you'd like to use it with I2C devices)
 *            
 *  - alternativ(!): arduino nano 
 *     -- arduino pin 11 & 12 dienen z.B. als ausgang fuer motorboard; pin 2 zwingend DCC command
 *        chinateil mit "Old Bootloader" (ATmega328p) programmieren; programmer AVRISP mkII
 *     -- mit weiteren motorboards kann der decoder fuer mehrere weichen erweitert werden: pin D3-D12 + A0-A7 ermoeglichen 9 weichen. 
 *        weichenlaternensteuerung ebenfalls durch ergenzung moeglich.
 *     -- spannungsregler LM 1117 MP5,0 ggf. auf eingangsspannung (thermalabschaltung) pruefen bzw. zusätzliche spnnungsregelung oder DC-DC-wandlung vornehmen.
 *               
 *            
 *  - Drv8871 DC motorboard (adafruit nachbau)
 *  - alternative für doppelspulenatrieb LGB 1206 mit passendem treiber (mind. 3A), z.B. darlington BD677 oder (fertigem) mosfet-board mit z.B. IRF520
 *  
 *  - netzteil: brueckengleichrichter, elko 10µF/25V, ggf. sicherung 2A, ggf. vorwiderstand Attiny85 100Ω/2W (47 - 200Ω)
 *  
 *  - vorwiderstand 22kΩ - 27kΩ fuer commandleitung
 *  
 *  - passt mit Attiny85 in Strapubox-MG-307SW modulgehaeuse 45x30x18 mm3
 *  
 *  - alternative schaltung via optokoppler (6N137) fuer commandleitung
 *     -- Dave Falkenburg: https://abload.de/img/schematic1haslf.png / arcomora: https://www.arcomora.com (diode: 4004)
 *     -- Rudysmodelrailway: https://rudysarduinoprojects.files.wordpress.com/2019/04/dcc-opto.png
 *     -- Philipp Gahtow: http://pgahtow.de/wiki/index.php?title=Datei:DCC_mini_Dekoder_sch.png
 *     -- bezug fertige 24V-optokoppler boards (stand april 2021): https://de.aliexpress.com/item/32914498721.html
 *     
 */

#include <NmraDcc.h> // nur version 1.4.2

#define DCC_DECODER_VERSION_ID 02 // versions-ID

uint32_t previousMillis1a = 0;
uint32_t previousMillis1b = 0;  

NmraDcc DCC;

void setup() 
{
   pinMode(ausgang1a, OUTPUT);     
   pinMode(ausgang1b, OUTPUT);
   pinMode(weichenlaterne1_pin, OUTPUT);
   
   DCC.init(MAN_ID_DIY, DCC_DECODER_VERSION_ID, FLAGS_OUTPUT_ADDRESS_MODE | FLAGS_DCC_ACCESSORY_DECODER,0);
   DCC.pin(0, 2, 1); // DCC-signal an pin2

   analogWrite (weichenlaterne1_pin, weichenlaterne1_dim); // wenn weichenlaterne bei einschalten leuchten soll
}

void loop() 
{
    
    DCC.process(); // analyse DCC-signal
    
    if (millis() - previousMillis1a > intervall) // routine schaltzeit
       {
        digitalWrite(ausgang1a, LOW);
       }
    if (millis() - previousMillis1b > intervall)
       {
        digitalWrite(ausgang1b, LOW);
       }
       
    //analogWrite (weichenlaterne1_pin, weichenlaterne1_dim);  // falls dauerleuchten der weichenlaterne gewuenscht
}

void notifyDccAccState(uint16_t Addr, uint16_t BoardAddr, uint8_t OutputAddr, uint8_t State)
//wird von DCC.process() aufgerufen, wenn ein weichenstellbefehl empfangen wurde
{        
       if (Addr == DCC_adresse)
        {
           if (OutputAddr & 0x1) 
              {  
                digitalWrite (ausgang1a, HIGH);
                previousMillis1a = millis();   // fuer schaltzeit
                digitalWrite (ausgang1b, LOW);
               }           
            else
              {  
                digitalWrite (ausgang1b, HIGH);
                previousMillis1b = millis();  // fuer schaltzeit
                digitalWrite (ausgang1a, LOW);
              }     
       } 

       if (Addr == weichenlaterne1_adr)
        {
           if (OutputAddr & 0x1) 
              {  
                digitalWrite (weichenlaterne1_pin, LOW); 
              }
            else
              {
                analogWrite (weichenlaterne1_pin, weichenlaterne1_dim); 
              }
         }       
}
